# Contributing

This document describes how we work and collaborate on this project.
It describes our workflow
and outlines the specific criteria that must be met
for a new increment of our application to be considered releasable.
It serves as a checklist for developers and AI agents to ensure quality and consistency in our releases.

## Workflow

We organize our work on Github, using our [github project].
AI agents can use the command line tool `gh` to access this project's issues.
The workflow is: Backlog > Next > Doing > Done.
Tip: use `gh issue --help` and `gh project --help` if you are not familiar with the `gh` cli.
Only work on a single item at a time.
For the active work item, organize our in [doing.md](./work/doing.md).
Use the following outline:

```text
# Doing
## Goal
## Analysis
## Design
## Steps
## Progress
```

## Code Quality

Our code is well-readable and understandable for humans and AI agents.
For this application, we prefer readability, simplicity, and understandability over performance.
Since we generate a lot of code, we do not do any additional linting ourselves.

Enforced on merges to `main` by [`ci.yml` Github action](.github/workflows/ci.yml).

Measure: `make quality`

### Code Style Guidelines

In addition to our flake8 configuration, adhere to the following guidelines:

1. **Conciseness**: Keep code and comments concise. Avoid unnecessary verbosity.
   - Use short, descriptive function and variable names
   - Minimize comments to what's truly necessary
   - Avoid redundant explanations

2. **Naming Conventions**:
   - Use descriptive names that indicate purpose

3. **Code Organization**:
   - Extract reusable functions to module level
   - Group related functionality together
   - Minimize nesting levels

4. **Documentation**:
   - Docstrings should be concise and focus on what's not obvious
   - For public APIs, include examples in docstrings
   - Avoid step-by-step comments within function bodies

5. **Simplicity**:
   - Prefer simple, straightforward implementations
   - Avoid complex parameter lists when possible
   - Default parameters should be used sparingly

## Testing

We validate our code by testing it.
Do not add code without tests.
Fast, local unit tests can be run without any prerequisites.
These tests are part of our CI job.

Measure: `make tests`

Our integration tests require access to a local database and a Floriday staging environment.
These tests ensure our client works end-to-end.
These tests as of now are not part of our CI job and need to be executed manually before creating a release.
Developers should run integration tests often during development and always before creating a release to ensure the client works correctly with the Floriday API.

Measure: `make test-integration`

## Documentation

Documentation in [`./docs`](./docs) is automatically generated by the swagger-codegen tool and should not be modified manually.
Meta-documentation should be placed in [`./about/`](./about/) directory, which will be created when needed.

## Performance and Security

For now, these require manual verification:

- [ ] No sensitive information (API keys, passwords) is hardcoded, logged, or printed to the terminal
- [ ] All database queries are optimized and indexed appropriately
- [ ] Environment variables are used for all configuration options

## DevOps

- [ ] Version number is incremented appropriately
- [ ] Configuration files e.g. (.flake8, .pre-commit-config.yaml, .vscode/settings.json) are up-to-date and committed to the repository

## Cross platform compatibility

- [ ] Application runs successfully on both development and production environments
- [ ] Any new dependencies are added to `pyproject.toml` and are compatible with both Linux and macOS

## Releasable (Done)

An increment is ready to be released when:

- There are no uncommitted changes
- All tests pass
- We can run example.py against the staging server successfully
- Open work is finished

Running `make release` will take care of:

- [x] Code and documentation are pushed to the central code repository
- [x] Release is identified using Semantic Versioning
- [x] Release is tagged in the repository
- [x] GitHub Actions package workflow successfully builds and publishes the package

If the package was successfully published, then the increment is Done

## Final Checks

- [ ] GitHub Actions workflow passes on the main branch, running all quality checks

## Undone for now

The following items are considered important for future releases but are not currently part of our Definition of Done:

- [ ] Implement test coverage measurement and maintain or improve coverage (verify with coverage tool)
- [ ] Create a script to automatically check if all environment variables in `.env.example` and `mongo/.env.default` are up to date
- [ ] Implement a zipped version of the code for download as part of the release process
- [ ] Test the application in both development and production environments.
- [ ] Document our application using specification by example. For now, we use simple `pytest`s.
- [ ] Cross platform compatibility
  - [ ] Linux
  - [ ] Mac OSX

This contributing guideline should be reviewed and updated periodically to reflect the evolving needs of the project.
Items from the "Undone for now" section should be considered for inclusion in the main DoD as they become implemented and integrated into our development process.

[github project]: https://github.com/orgs/serraict/projects/3/
